// $Id: pdfclock.m,v 1.4.2.1 2013/12/02 17:47:04 rjs Exp $
//
//  pdfclock.m
//  A little PDFlib application to draw an analog clock.
//  This time using native C-Code in a Objective-C class
//


#import "pdfclock.h"

#include <stdio.h>
#include <time.h>
#include <stdlib.h>

#if TARGET_OS_IPHONE
#import "PDFlib_ios/PDFlib_objc.h"
#else
#import "PDFlib/PDFlib_objc.h"
#endif
#define RADIUS          200.0
#define MARGIN          20.0

@implementation pdfclock

+ (NSData *) run {
      
    NSData *pdfdata=nil;
    

    PDF         *p;
    double      alpha;
    time_t      timer;
    struct tm   ltime;
    const char *buf;
    long size;

    /* create a new PDFlib object */
    if ((p = PDF_new()) == (PDF *) 0)
    {
        return pdfdata;
    }

    PDF_TRY(p) {
        PDF_set_option(p, "errorpolicy=exception");

        PDF_begin_document(p, "", 0, "");
        PDF_set_info(p, "Creator", "pdfclock.c");
        PDF_set_info(p, "Author", "Thomas Merz");
        PDF_set_info(p, "Title", "PDF clock (C)");
        
        PDF_begin_page_ext(p, 2 * (RADIUS + MARGIN), 2 * (RADIUS + MARGIN), "");
        
        PDF_translate(p, RADIUS + MARGIN, RADIUS + MARGIN);
        PDF_setcolor(p, "fillstroke", "rgb", 0, 0, 1, 0);
        PDF_save(p);

        /* minute strokes */
        PDF_setlinewidth(p, 2);
        for (alpha = 0; alpha < 360; alpha += 6)
        {
            PDF_rotate(p, 6);
            PDF_moveto(p, RADIUS, 0);
            PDF_lineto(p, RADIUS - MARGIN/3, 0);
            PDF_stroke(p);
        }

        PDF_restore(p);
        PDF_save(p);

        /* 5 minute strokes */
        PDF_setlinewidth(p, 3);
        for (alpha = 0; alpha < 360; alpha += 30)
        {
            PDF_rotate(p, 30);
            PDF_moveto(p, RADIUS, 0);
            PDF_lineto(p, RADIUS-MARGIN, 0);
            PDF_stroke(p);
        }

        time(&timer);
        ltime = *localtime(&timer);

        /* draw hour hand */
        PDF_save(p);
        PDF_rotate(p, -((ltime.tm_min/60.0) + ltime.tm_hour - 3.0) * 30.0);
        PDF_moveto(p, -RADIUS/10, -RADIUS/20);
        PDF_lineto(p, RADIUS/2, 0);
        PDF_lineto(p, -RADIUS/10, RADIUS/20);
        PDF_closepath(p);
        PDF_fill(p);
        PDF_restore(p);

        /* draw minute hand */
        PDF_save(p);
        PDF_rotate(p, -((ltime.tm_sec/60.0) + ltime.tm_min - 15.0) * 6.0);
        PDF_moveto(p, -RADIUS/10, -RADIUS/20);
        PDF_lineto(p, RADIUS * 0.8, 0);
        PDF_lineto(p, -RADIUS/10, RADIUS/20);
        PDF_closepath(p);
        PDF_fill(p);
        PDF_restore(p);

        /* draw second hand */
        PDF_setcolor(p, "fillstroke", "rgb", 1, 0, 0, 0);
        PDF_setlinewidth(p, 2);
        PDF_save(p);
        PDF_rotate(p, -((ltime.tm_sec - 15.0) * 6.0));
        PDF_moveto(p, -RADIUS/5, 0);
        PDF_lineto(p, RADIUS, 0);
        PDF_stroke(p);
        PDF_restore(p);

        /* draw little circle at center */
        PDF_circle(p, 0, 0, RADIUS/30);
        PDF_fill(p);

        PDF_restore(p);

        PDF_end_page_ext(p, "");
        PDF_end_document(p, "");

        buf = PDF_get_buffer(p, &size);
        pdfdata = [NSData dataWithBytes:buf length:size];
    }

    PDF_CATCH(p) {
        char buf[512];
        sprintf(buf, "[%d] %s: %s\n", PDF_get_errnum(p), PDF_get_apiname(p), PDF_get_errmsg(p));
     
#if TARGET_OS_IPHONE
        UIAlertView *alert = [[UIAlertView alloc] 
            initWithTitle:@"PDFlib exception"
            message:[NSString stringWithUTF8String:buf]
            delegate:self cancelButtonTitle:nil otherButtonTitles:@"OK" , nil];
        [alert show];
#else
        NSAlert *alert = [[NSAlert alloc] init];
        [alert setMessageText:@(buf)];
        [alert runModal];
#endif
    }

    PDF_delete(p);

    return pdfdata;
}

@end
